@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -26,11 +26,43 @@
 #endif
 
 #include "modsecurity.h"
+#include "re.h"
+
+#ifdef WIN32
+#include <ws2tcpip.h>
+// This is a trick: for ModSecurity modules this will declare inet_pton,
+// but for mymodule.cpp (IIS module) this will skip, because we include
+// windows.h before including msc_util.h
+// Without the trick we have redefinition conflict.
+//
+#if !(NTDDI_VERSION >= NTDDI_VISTA)
+int DSOLOCAL inet_pton(int family, const char *src, void *dst);
+#endif
+#endif
+
+#define UNICODE_ERROR_CHARACTERS_MISSING    -1
+#define UNICODE_ERROR_INVALID_ENCODING      -2
+#define UNICODE_ERROR_OVERLONG_CHARACTER    -3
+#define UNICODE_ERROR_RESTRICTED_CHARACTER  -4
+#define UNICODE_ERROR_DECODING_ERROR        -5
+
+#ifdef LINUX_S390
+int DSOLOCAL swap_int32(int x);
+#endif
+
+
+char DSOLOCAL *utf8_unicode_inplace_ex(apr_pool_t *mp, unsigned char *input, long int input_len, int *changed);
+
+char DSOLOCAL *m_strcasestr(const char *haystack, const char *needle);
 
 int DSOLOCAL normalize_path_inplace(unsigned char *input, int len, int win, int *changed);
 
 int DSOLOCAL parse_boolean(const char *input);
 
+char DSOLOCAL *remove_quotes(apr_pool_t *mptmp, const char *input, int input_len);
+
+char DSOLOCAL *parse_pm_content(const char *op_parm, unsigned short int op_len, msre_rule *rule, char **error_msg);
+
 char DSOLOCAL *remove_escape(apr_pool_t *mptmp, const char *input, int input_len);
 
 int DSOLOCAL parse_name_eq_value(apr_pool_t *mp, const char *input, char **name, char **value);
@@ -59,9 +91,9 @@ char DSOLOCAL *current_logtime(apr_pool_t *mp);
 
 char DSOLOCAL *current_filetime(apr_pool_t *mp);
 
-int DSOLOCAL msc_mkstemp_ex(char *template, int mode);
+int DSOLOCAL msc_mkstemp_ex(char *templat, int mode);
 
-int DSOLOCAL msc_mkstemp(char *template);
+int DSOLOCAL msc_mkstemp(char *templat);
 
 char DSOLOCAL *strtolower_inplace(unsigned char *str);
 
@@ -111,4 +143,7 @@ char DSOLOCAL *construct_single_var(modsec_rec *msr, char *name);
 
 char DSOLOCAL *format_all_performance_variables(modsec_rec *msr, apr_pool_t *mp);
 
+unsigned char DSOLOCAL is_netmask_v4(char *ip_strv4);
+
+unsigned char DSOLOCAL is_netmask_v6(char *ip_strv6);
 #endif
